rand('seed', 1);

% general parameters
src = 'dm11234';                         % which source to use (claps or three char code,gender,num,sen_num (sentences) or d,gender,number (digits))
dsr = 16000;                             % desired sampling rate
do_env_plot = 0;                         % show plots of frequency bands and their envelopes
do_mask_plot = 1;                        % show a summary plot of the masks
do_result_plot = 0;                      % show waveform results
time_plots = 0;                          % if 1, show time signals, else show (log)spectrograms
save_wav = 1;                            % save sounds results to wav files
min_db = -400;                           % minimum allowed dB value

% evelope follower parameters
ef_type = 'hilbert';                     % type of envelope follower to use
f_cut = 80;                              % cutoff freq (Hz) of the EF's LPF
flpo = 1;                                % order of the EF's LPF (careful to change if ef_type changes)

% analysis parameters
win_sz = 1600;                         % length of the analysis window (in samples, default=0.1sec)
mask_tol = .015;                         % tolerance for error between d_est and denvr_est when creating mask (larger=throw more away)
mask_fo = dsr/20;                        % mask low-pass filter order
mask_ff = 1;                         % mask low-pass filter frequency (lower=smoother)
gamma = 0.25;                            % percentile to use when estimating the log-envelope slope
bpo = 3;                                 % bins per octave to use in the filterbank
bpfo = 3;                                % filterbank filter order
min_freq = 80;                           % minimum frequency (Hz) to use in the filterbank
rev_seq = 1;                             % use filtfilt instead of filter in filterbank calcs
filt_type = 'logbutter';                 % filter type to use in filterbank ('logbutter', linbutter', 'gammatone', 'fir')

% reverberation parameters
rev_type = 'fir';                        % type of reverb to use ('fir', 'file')
rev_file = 'Lexicon_480L-SmallStage2';   % IR file to use if rev_type = 'file'
T60 = 1.0;                               % the T60 value (sec) to use when generating artificial reverb (rev_type='fir')

params = [src '_winsz' num2str(win_sz) '_tol' num2str(mask_tol) '_ff' num2str(mask_ff) '_bpo' num2str(bpo) '_bpfo' num2str(bpfo) '_' filt_type '_T60' num2str(T60)];

% load the clean source file
switch src
    case 'claps'
        [yr,sr] = wavread('sound/claps.wav');
        yr = yr ./ (2*max(abs(yr)));
        yc = yr;
        % resample if needed
        if sr ~= dsr
            yr = resample(yr, dsr, sr);
        end
        
    case 'aclaps'
        [yc,sr] = wavread('sound/aclaps.wav');
        
    case 'drums'
        [yc,sr] = wavread('sound/drums.wav');
        yc = [randn(sr/5,1)*.001 ; yc];
        yc = yc ./ (2*max(abs(yc)));
        
    otherwise
        if src(1)=='d'
            [yc,sr] = wavread(['sound/digits/' src(2:3) '_' src(4:end) '.wav']);
        else
            [yc,sr] = wavread(['sound/' src(1:2) '/sa' src(3) '.wav']);
        end
end

% resample if needed
if sr ~= dsr
    yc = resample(yc, dsr, sr);
    sr = dsr;
end

% normalize so that clean signal has max amp of 0.5
yc = yc ./ (2*max(abs(yc)));

% create/load the reverberated sound
if ~strcmp(src, 'claps')
    switch rev_type
        case 'fir'  % generate artificial impulse response
            hr = rev_fir(T60,sr);
            
        case 'file' % load a sound file
            [hr,hsr] = wavread(['IR/' rev_file '.wav']);
            hr = hr(:,1); % make mono
            if hsr > dsr
                hr = resample(hr, dsr, hsr);
            end
            
        otherwise
            error('unknown rev_type!');
    end
    
    % filter sound with IR
    yr = filter(hr,1,yc);
    
    % normalize
    yr = yr' ./ (2*max(abs(yr)));
end

% filterbanks
[yrb,frqs] = filterbank(yr, filt_type, sr, bpo, min_freq, bpfo, rev_seq);
[ycb,frqs] = filterbank(yc, filt_type, sr, bpo, min_freq, bpfo, rev_seq);

[nchan,N] = size(yrb);

denvr_est = zeros(nchan,N);
envr = zeros(nchan,N);
envc = zeros(nchan,N);
mask_est = ones(nchan,N);
mask_rec = ones(nchan,N);
d_est = zeros(nchan,1);
T60_est = zeros(nchan,1);
tau_est = zeros(nchan,1);
flen_est = zeros(nchan,1);
env_est = zeros(nchan,1);

% do per-channel analysis
for c = 1:nchan
    % calculate envelope
    envr(c,:) = calc_env(yrb(c,:), sr, ef_type, flpo, f_cut);
    envc(c,:) = calc_env(ycb(c,:), sr, ef_type, flpo, f_cut);
    
    % estimate envelope slope on per-window basis, (dB/sample)
    x = [[1:win_sz]'  ones(win_sz,1)];
    envrd = mag2db(envr(c,:));
    for i = win_sz:N
        win = envrd(i-win_sz+(1:win_sz))';
        win(win < min_db) = min_db;
        p = x\win;
        if isnan(p(1))
            warning(['nan at pos ' num2str(i) ' in channel ' num2str(c)]);
        end
        denvr_est(c,i) = p(1);
    end
    
    % order-statistics
    ndenvr = denvr_est(c,denvr_est(c,:) < 0); % only keep negative slope ests
    [a,b] = hist(ndenvr, length(ndenvr));
    ndx = find(cumsum(a)/sum(a) > gamma);
    ndx = ndx(1);
    d_est(c) = b(ndx);
    T60_est(c) = -60/d_est(c)/sr;
    tau_est(c) = T60_est(c)/log(1000);
    flen_est(c) = round(T60_est(c)*sr);
    %env_est(c) = exp(-([0:flen_est(c)-1]/sr)/tau_est(c));
    
    % decide what to THROW AWAY by making appropriate parts of mask 0
    lp = fir1(20, 1e-2);
    denvr_est(c,:) = filter(lp, 1, denvr_est(c,:));
    denvr_est(c,:) = shift(denvr_est(c,:), -10);
    %[B,A] = butter(1, 10e-3);
    %denvr_est(c,:) = filter(B,A, denvr_est(c,:));
    
    if any(isnan(denvr_est(c,:)))
        warning(['nan at channel ' num2str(c)]);
    end
    
    %mask_est(c, abs(denvr_est(c,:)-d_est(c))<mask_tol & denvr_est(c,:)<0) = 0;
    mask_est(c, abs(denvr_est(c,:)-d_est(c)) < mask_tol) = 0;
    mask_rec(c, envr(c,:)./envc(c,:) > 2) = 0;
end

% shift mask to compensate for delay
mask_est = shift(mask_est, -round(win_sz/2));

% create the reconstructed clean signal
yc_est = zeros(1,N);
yc_rec = zeros(1,N);
yc_oest = zeros(1,N);

if mask_ff > 0
    %[B,A] = butter(mask_fo, mask_ff);
    B = ones(1,mask_fo)/mask_fo;
    A = 1;
end

maskf_est = mask_est;
maskf_rec = mask_rec;
for c = 1:nchan
    if mask_ff > 0
        % low-pass filter the mask
        maskf_est(c,:) = filtfilt(B,A,mask_est(c,:));
        maskf_rec(c,:) = filtfilt(B,A,mask_rec(c,:));
        %maskf(c,:) = filter(B,A,mask_est(c,:));
    end
    
    % reference reconstruction
    yc_rec = yc_rec + ycb(c,:);
    
    % weight original signal by mask values
    yc_est = yc_est + yrb(c,:) .* maskf_est(c,:);
    yc_oest = yc_oest + yrb(c,:) .* maskf_rec(c,:);
end

% normalize
yc_rec = yc_rec ./ (2*max(yc_rec));
yc_est = yc_est ./ (2*max(yc_est));
yc_oest = yc_oest ./ (2*max(yc_oest));

% plotting stuff
if do_env_plot
    for c = 1:nchan
        figure;
        subplot(411);
        plot(ycb(c,:));
        axis tight;
        title(['clean signal, f = ' num2str(frqs(c)) ' ' num2str(frqs(c+1))]);
        subplot(412);
        plot(yrb(c,:));
        hold on;
        plot(envr(c,:), 'r-');
        tmp = nan*ones(1,N);
        tmp(find(mask_est(c,:)==1))=0;
        plot(tmp, 'g-', 'linewidth', 3);
        axis tight
        title('reverberant signal, envelope, and mask');
        subplot(413);
        plot(denvr_est(c,:));
        axis tight;
        title(['denvrest, dest = ' num2str(d_est(c))]);
        subplot(414);
        hist(denvr_est(c,:),N/100);
        axis tight;
        title(['histogram of denvrest, dest = ' num2str(d_est(c))]);
    end
end

if do_result_plot
    lfs_rec = max(20*log10(logfsgram(yc_rec, 512, sr, 128, 64, min_freq, bpo)), min_db/2);
    lfs_est = max(20*log10(logfsgram(yc_est, 512, sr, 128, 64, min_freq, bpo)), min_db/2);
    clim = [min([lfs_rec(:) ;lfs_est(:)]) max([lfs_rec(:) ;lfs_est(:)])];
    
    figure;
    subplot(311);
    plot(yr);
    title('reverberant');
    
    subplot(312);
    if time_plots
        plot(yc_rec);
    else
        switch filt_type
            case 'gammatone'
                spectrogram(yc_rec,  256, 128, 512, sr, 'yaxis');
                
            otherwise
                plotlogfsgram(yc_rec, lfs_rec, sr, nchan, frqs, clim);
        end
    end
    title('recon clean');
    
    subplot(313);
    if time_plots
        plot(yc_est);
    else
        switch filt_type
            case 'gammatone'
                spectrogram(yc_est,  256, 128, 512, sr, 'yaxis');
                
            otherwise
                plotlogfsgram(yc_est, lfs_est, sr, nchan, frqs, clim);
        end
    end
    title('estimated clean');
end

if do_mask_plot
    eval(['mkdir results/' params]);
    
    lfs_r = max(20*log10(logfsgram(yr, 512, sr, 128, 64, min_freq, bpo)), min_db/4);
    lfs_rec = max(20*log10(logfsgram(yc_rec, 512, sr, 128, 64, min_freq, bpo)), min_db/4);
    lfs_est = max(20*log10(logfsgram(yc_est, 512, sr, 128, 64, min_freq, bpo)), min_db/4);
    lfs_oest = max(20*log10(logfsgram(yc_oest, 512, sr, 128, 64, min_freq, bpo)), min_db/4);
    clim = [min([lfs_rec(:) ; lfs_est(:) ; lfs_r(:) ; lfs_oest(:)]) max([lfs_rec(:) ; lfs_est(:) ; lfs_r(:) ; lfs_oest(:)])];
    
    figure;
    subplot(611);
    plotlogfsmask(mask_est, sr, nchan, frqs);
    title('Estimated Mask');
    
    subplot(612);
    plotlogfsmask(mask_rec, sr, nchan, frqs);
    title('Ideal Mask');
    
    subplot(613);
    if time_plots
        plot(yr);
    else
        switch filt_type
            case 'gammatone'
                spectrogram(yr, 256, 128, 512, sr, 'yaxis');
                
            otherwise
                plotlogfsgram(yr, lfs_r, sr, nchan, frqs, clim);
        end
    end
    title('Reverberant');
    axis tight;
    
    subplot(614);
    if time_plots
        plot(yc_rec);
    else
        switch filt_type
            case 'gammatone'
                spectrogram(yc_rec,  256, 128, 512, sr, 'yaxis');
                
            otherwise
                plotlogfsgram(yc_rec, lfs_rec, sr, nchan, frqs, clim);
        end
    end
    title('Clean (reconstructed)');
    axis tight;
    
    subplot(615);
    if time_plots
        plot(yc_oest);
    else
        switch filt_type
            case 'gammatone'
                spectrogram(yc_oest,  256, 128, 512, sr, 'yaxis');
                
            otherwise
                plotlogfsgram(yc_oest, lfs_oest, sr, nchan, frqs, clim);
        end
    end
    title('Oracle Estimated Clean');
    axis tight;
    
    subplot(616);
    if time_plots
        plot(yc_est);
    else
        switch filt_type
            case 'gammatone'
                spectrogram(yc_est,  256, 128, 512, sr, 'yaxis');
                
            otherwise
                plotlogfsgram(yc_est, lfs_est, sr, nchan, frqs, clim);
        end
    end
    title('Estimated Clean');
    axis tight;
    colormap jet;
    
    % export generated figures to eps files
    exportfig(gcf, ['results/' params '/results.eps'], 'Width', 40, 'Height', 30, 'FontMode', 'fixed', 'FontSize', 10, 'Color', 'rgb');
    
    figure;
    plotlogfsmask(mask_est, sr, nchan, frqs);
    title('Estimated Mask');
    set(gcf, 'Position', [0 0 1280 200])
    exportfig(gcf, ['results/' params '/mask_est.eps'], 'Width', 15, 'Height', 2, 'FontMode', 'fixed', 'FontSize', 10);
    
    figure;
    plotlogfsmask(mask_rec, sr, nchan, frqs);
    title('Ideal Mask');
    set(gcf, 'Position', [0 0 1280 200])
    exportfig(gcf, ['results/' params '/mask_rec.eps'], 'Width', 15, 'Height', 2, 'FontMode', 'fixed', 'FontSize', 10);
    
    figure;
    plotlogfsgram(yr, lfs_r, sr, nchan, frqs, clim);
    title('Reverberant Signal');
    set(gcf, 'Position', [0 0 1280 200])
    exportfig(gcf, ['results/' params '/yr.eps'], 'Width', 15, 'Height', 2, 'FontMode', 'fixed', 'FontSize', 10, 'Color', 'rgb');
    
    figure;
    plotlogfsgram(yc_rec, lfs_rec, sr, nchan, frqs, clim);
    title('Clean (reconstructed)');
    set(gcf, 'Position', [0 0 1280 200])
    exportfig(gcf, ['results/' params '/yc_rec.eps'], 'Width', 15, 'Height', 2, 'FontMode', 'fixed', 'FontSize', 10, 'Color', 'rgb');
    
    figure;
    plotlogfsgram(yc_oest, lfs_oest, sr, nchan, frqs, clim);
    title('Oracle Estimated Clean');
    set(gcf, 'Position', [0 0 1280 200])
    exportfig(gcf, ['results/' params '/yc_oest.eps'], 'Width', 15, 'Height', 2, 'FontMode', 'fixed', 'FontSize', 10, 'Color', 'rgb');
    
    figure;
    plotlogfsgram(yc_est, lfs_est, sr, nchan, frqs, clim);
    title('Algorithm Estimated Clean');
    set(gcf, 'Position', [0 0 1280 200])
    exportfig(gcf, ['results/' params '/yc_est.eps'], 'Width', 15, 'Height', 2, 'FontMode', 'fixed', 'FontSize', 10, 'Color', 'rgb');
    
    close all;
end

% save the results
if save_wav
    wavwrite(yc_rec, sr, ['results/' params '/y_clean_rec.wav']);
    wavwrite(yr, sr, ['results/' params '/y_reverb.wav']);
    wavwrite(yc_est, sr, ['results/' params '/y_clean_est.wav']);
    wavwrite(yc_oest, sr, ['results/' params '/y_clean_oest.wav']);
end

% play the results
eval(['!totem results/' params '/y_clean_rec.wav' ' results/' params '/y_reverb.wav' ' results/' params '/y_clean_est.wav' ' results/' params '/y_clean_oest.wav'])

% don't include first win_sz samples in ideal mask comparison
ok_mask_rec = mask_rec(:,win_sz+1:end);
ok_mask_est = mask_est(:,win_sz+1:end);

% some evaluation metrics
acc = sum(ok_mask_rec(:)==ok_mask_est(:))/numel(ok_mask_rec);
snr_est = snr(yc_est, yc_rec);
snr_oest = snr(yc_oest, yc_rec);

% save evaluation results to a mat file
eval(['save ' 'results/' params '/results.mat acc snr_est snr_oest']);


